package prefuse.controls;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.util.Iterator;

import javax.swing.SwingUtilities;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.data.tuple.TupleSet;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import cytoPrefuse.CytoPrefuse;
import cytoPrefuse.EdgeBundler;

public class GridDragger extends ControlAdapter {

	protected Point2D down = new Point2D.Double();
    protected Point2D temp = new Point2D.Double();
    protected boolean dragged, resetItem;
    private Visualization m_vis;
    private TupleSet draggableItems;
    private EdgeBundler edgeBundler;
    /**
     * Creates a new drag control that issues repaint requests as an item
     * is dragged.
     * @param searchGridLayout 
     */
    public GridDragger(Visualization vis, TupleSet draggableItems, EdgeBundler edgeBundler) {
    	    	
 	   m_vis = vis;
 	   this.draggableItems = draggableItems;
 	   this.edgeBundler = edgeBundler;
    }
    
    
    /**
     * @see prefuse.controls.Control#itemEntered(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemEntered(VisualItem item, MouseEvent e) {
	    if (item instanceof NodeItem){
	        Display d = (Display)e.getSource();
	        d.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));      
	    }
      
    }
    
    /**
     * @see prefuse.controls.Control#itemExited(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemExited(VisualItem item, MouseEvent e) {
        if (item instanceof NodeItem){ 
	        Display d = (Display)e.getSource();
	        d.setCursor(Cursor.getDefaultCursor());
        }
    } //
    
    /**
     * @see prefuse.controls.Control#itemPressed(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemPressed(VisualItem item, MouseEvent e) {
    	if (item instanceof NodeItem){
	        if (!SwingUtilities.isLeftMouseButton(e)) return;
	        dragged = false;
	        Display d = (Display)e.getComponent();
	        d.getAbsoluteCoordinate(e.getPoint(), down);
	        Iterator it = draggableItems.tuples();
	        while (it.hasNext()){
	        	VisualItem vi = (VisualItem) it.next();
	        	vi.setStartX(vi.getX());
	        	vi.setEndX(vi.getX());            
	        	vi.setStartY(vi.getY());
	        	vi.setEndY(vi.getY());
	        }
    	}
    }
    
    /**
     * @see prefuse.controls.Control#itemReleased(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemReleased(VisualItem item, MouseEvent e) {
    	if (item instanceof NodeItem){
	    	if (!SwingUtilities.isLeftMouseButton(e)) return;
	        if ( dragged ) {
        		edgeBundler.recomputeLocations();
        		m_vis.run("aggLayout");
    	        CytoPrefuse cytoPrefuse = (CytoPrefuse)m_vis.getDisplay(0);
    	        cytoPrefuse.labelLayout.setLabelPosition();
	        }
    	}
    }
 
	/**
     * @see prefuse.controls.Control#itemDragged(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemDragged(VisualItem item, MouseEvent e) {
    	if (item instanceof NodeItem){
	    	if (!SwingUtilities.isLeftMouseButton(e)) return;
	        TupleSet selectedItems = m_vis.getFocusGroup(CytoPrefuse.SELECTED);
	    	 
		    if (!selectedItems.containsTuple(item)) {
		        boolean shift = e.isShiftDown();
	            if ( !shift ) {
	            	selectedItems.setTuple(item);
	            } else {
	            	selectedItems.addTuple(item);
	            }
		    }
	        dragged = true;
	        Display d = (Display)e.getComponent();
	        d.getAbsoluteCoordinate(e.getPoint(), temp);
	        double dx = temp.getX()-down.getX();
	        double dy = temp.getY()-down.getY();
	           
       	     
			Iterator selectedIt = selectedItems.tuples();
	        while (selectedIt.hasNext()) {
	        	VisualItem vi = (VisualItem)selectedIt.next();
	        	if (vi instanceof NodeItem) {
	        		NodeItem nodeItem = (NodeItem) vi;
	        	    double x = nodeItem.getX();
	       	        double y = nodeItem.getY();			 	    
	       	     
			        nodeItem.setStartX(x);  
			        nodeItem.setStartY(y);
			        nodeItem.setX(x+dx); 
			        nodeItem.setY(y+dy);
			        nodeItem.setEndX(x+dx);
			        nodeItem.setEndY(y+dy);			        
	        	
		        }
		        CytoPrefuse cytoPrefuse = (CytoPrefuse)m_vis.getDisplay(0);
		        cytoPrefuse.labelLayout.setLabelPosition();
				m_vis.repaint();
	        }    
	        
	        down.setLocation(temp);	       
    	}
   }

   

} // end of class Drag